#include <math.h>
#include <allegro.h>
#include "alleggl.h"
#include "sound.h"
#include "circle.h"
#include "utils.h"
#include "map.h"
#include "init.h"
#include "triangles.h"

int						frames = 0;
volatile int			secs;
int						fog_mode = 2; // 0 = exp, 1 = exp2, 2 = linear
int						keyboard_action = 0; // 0 = for fast comps, 1 for slow comps
CAMERA					camera;
int						screen_width, screen_height, color_depth, full_screen;
int						selecty;
BITMAP					*intro;

void set_camera_position(void);
void program_exit(void);
void init_fog(void);
void init_gl(void);
int program_init(void);
void secs_timer(void);

void set_camera_position(void)
{
	allegro_gl_begin();
	glMatrixMode (GL_PROJECTION);
	glLoadIdentity ();
	glFrustum(-1.6, 1.6, -1.2, 1.2, 2, 80.0);
	glMatrixMode (GL_MODELVIEW);
	glLoadIdentity ();
	glTranslatef(0, 0, -camera.dist);
	glRotatef(camera.xangle, 1, 0, 0);
	glRotatef(camera.zangle, 0, 0, 1);
	glTranslatef (-camera.translatex, camera.translatey, -camera.translatez);

	allegro_gl_end();
}

void program_exit(void)
{
	destroy_trianglelists();
	
/*
	set_gfx_mode (GFX_TEXT, 0, 0, 0, 0);
	allegro_message("Frames: %i, Seconds: %i, FPS: %f\n", frames, secs, (float)frames / (float)secs);
*/
}

void init_fog(void)
{
	float fogColor[4] = { 0.0, 0.0, 0.05, 1.0 };
	
	glEnable (GL_FOG);

	glFogfv(GL_FOG_COLOR, fogColor);

	if (fog_mode == 0)
	{
		glFogi(GL_FOG_MODE, GL_EXP);
		glFogf(GL_FOG_DENSITY, 0.06);
	}
	if (fog_mode == 1)
	{
		glFogi(GL_FOG_MODE, GL_EXP2);
		glFogf(GL_FOG_DENSITY, 0.04);
	}
	if (fog_mode == 2)
	{
		glFogi(GL_FOG_MODE, GL_LINEAR);
		glFogf(GL_FOG_START, 2.0);
		glFogf(GL_FOG_END, 50.0);
	}
}

void init_gl(void)
{
	install_allegro_gl();

	allegro_gl_clear_settings();
	allegro_gl_set (AGL_Z_DEPTH, 16);
	allegro_gl_set (AGL_DOUBLEBUFFER, 1);
	allegro_gl_set (AGL_STENCIL_DEPTH, 0);	
	allegro_gl_set (AGL_RENDERMETHOD, 1);
	allegro_gl_set (AGL_WINDOWED, TRUE);	
	allegro_gl_set (AGL_SUGGEST, AGL_Z_DEPTH | AGL_DOUBLEBUFFER | AGL_STENCIL_DEPTH
			| AGL_RENDERMETHOD | AGL_WINDOWED);
}

void show_start_menu(void)
{
	BITMAP *vscreen;
	int color;

	vscreen=create_bitmap(640,480);	
	text_mode(-1);
	//clear(vscreen);
	blit(intro,vscreen,0,0,0,0,640,480);
	textout(vscreen,font,"Speedhack 2002 Panic - Configuration",170,10,makecol(255,255,255));
	textout(vscreen,font,"Press ENTER to continue or ESCAPE to quit.",140,460,makecol(255,255,255));
	textout(vscreen,font,"Use the cursor keys up/down/left/right to make a selection.",80,440,makecol(255,255,255));
	textout(vscreen,font,"(Note: some video cards only support 16 bit colors in Fullscreen)",60,400,makecol(255,255,255));

	if (full_screen==FALSE)
		color=makecol(31,255,31);
	else
		color=makecol(191,191,191);
	textout(vscreen,font,"Windowed",200,60,color);
	if (full_screen==TRUE)
		color=makecol(31,255,31);
	else
		color=makecol(191,191,191);
	textout(vscreen,font,"Fullscreen",340,60,color);

	if (screen_width==320 && screen_height==240)
		color=makecol(31,255,31);
	else
		color=makecol(191,191,191);
	textout(vscreen,font,"320 x 240",200,100,color);
	if (screen_width==640 && screen_height==480)
		color=makecol(31,255,31);
	else
		color=makecol(191,191,191);
	textout(vscreen,font,"640 x 480",340,100,color);
	if (screen_width==800 && screen_height==600)
		color=makecol(31,255,31);
	else
		color=makecol(191,191,191);
	textout(vscreen,font,"800 x 600",200,120,color);
	if (screen_width==1024 && screen_height==768)
		color=makecol(31,255,31);
	else
		color=makecol(191,191,191);
	textout(vscreen,font,"1024 x 768",340,120,color);
	if (screen_width==1280 && screen_height==1024)
		color=makecol(31,255,31);
	else
		color=makecol(191,191,191);
	textout(vscreen,font,"1280 x 1024",200,140,color);
	if (screen_width==1600 && screen_height==1200)
		color=makecol(31,255,31);
	else
		color=makecol(191,191,191);
	textout(vscreen,font,"1600 x 1200",340,140,color);

	if (color_depth==16)
		color=makecol(31,255,31);
	else
		color=makecol(191,191,191);
	textout(vscreen,font,"16 bit colors",150,180,color);
	if (color_depth==32)
		color=makecol(31,255,31);
	else
		color=makecol(191,191,191);
	textout(vscreen,font,"32 bit colors",290,180,color);
	if (color_depth==0)
		color=makecol(31,255,31);
	else
		color=makecol(191,191,191);
	textout(vscreen,font,"desktop",440,180,color);

	textout(vscreen,font,"brightness",100,220,makecol(191,191,191));
	rect(vscreen,199,216,540,230,makecol(191,191,191));
	if (brightness<0) brightness=0;
	rectfill(vscreen,200,217,200+(int)((539-200)*brightness),229,makecol(31,191,31));

	if (selecty==0)
		rect(vscreen,60,50,580,76,makecol(255,255,255));
	if (selecty==1)
		rect(vscreen,60,90,580,156,makecol(255,255,255));
	if (selecty==2)
		rect(vscreen,60,170,580,196,makecol(255,255,255));
	if (selecty==3)
		rect(vscreen,60,210,580,236,makecol(255,255,255));

	acquire_bitmap(screen);
	blit(vscreen,screen,0,0,0,0,640,480);
	release_bitmap(screen);
	destroy_bitmap(vscreen);
}


void create_intro(void)
{
	int x,y,bright;

	intro=create_bitmap(640,480);	
	for (y=0; y<480; y++)
		for (x=0; x<640; x++)
		{
			bright=1500-sqrt((320-x)*(320-x)+(240-y)*(240-y))*2;
			putpixel(intro,x,y,makecol(
				((31+(rand() % 8))*bright)/1500,
				((31+(rand() % 8))*bright)/1500,
				((127+(rand() % 8))*bright)/1500
				));
		}
}

int start_menu(void)
{
	PACKFILE *fp;

	screen_width=640;
	screen_height=480;
	color_depth=16;
	full_screen=TRUE;
	selecty=0;
	set_color_depth(16);
	if (os_type==OSTYPE_WINXP || os_type==OSTYPE_WIN2000 || os_type==OSTYPE_WINNT ||
		os_type==OSTYPE_WINME || os_type==OSTYPE_WIN98 || os_type==OSTYPE_WIN95)
	{
		// start windowed, else color depth for desktop doesnt work
		if (set_gfx_mode(GFX_AUTODETECT, 640,480, 0, 0) < 0) {
			allegro_message ("Error setting graphics mode:\n%s\n", allegro_error);
			return -1;
		}
	} else
	{
		// safe
		if (set_gfx_mode(GFX_AUTODETECT, 640,480, 0, 0) < 0) {
			allegro_message ("Error setting graphics mode:\n%s\n", allegro_error);
			return -1;
		}
	}

	if (file_exists("paniccfg.dat", FA_ARCH, NULL))
	{
		fp=pack_fopen("paniccfg.dat",F_READ_PACKED);
		pack_fread(&screen_width, sizeof(int), fp);
		pack_fread(&screen_height, sizeof(int), fp);
		pack_fread(&color_depth, sizeof(int), fp);
		pack_fread(&full_screen, sizeof(int), fp);
		pack_fread(&brightness, sizeof(double), fp);
		pack_fclose(fp);
	}

	create_intro();

	show_start_menu();

	do
	{
		if (key[KEY_UP] && selecty>0)
		{
			selecty--;
			show_start_menu();
			do { } while (key[KEY_UP]);
		}
		if (key[KEY_DOWN] && selecty<3)
		{
			selecty++;
			show_start_menu();
			do { } while (key[KEY_DOWN]);
		}
		if (key[KEY_RIGHT])
		{
			if (selecty==0)
			{
				if (full_screen==FALSE)
					full_screen=TRUE;
				else
					full_screen=FALSE;
			}
			if (selecty==1)
			{
				if (screen_height==240)
				{ screen_width=640; screen_height=480; }
				else if (screen_height==480)
				{ screen_width=800; screen_height=600; }
				else if (screen_height==600)
				{ screen_width=1024; screen_height=768; }
				else if (screen_height==768)
				{ screen_width=1280; screen_height=1024; }
				else if (screen_height==1024)
				{ screen_width=1600; screen_height=1200; }
				else
				{ screen_width=320; screen_height=240; }
			}
			if (selecty==2)
			{
				if (color_depth==16)
					color_depth=32;
				else if (color_depth==32)
					color_depth=0;
				else
					color_depth=16;
			}
			if (selecty==3)
			{
				if (brightness<1) brightness+=0.05;
			}
			show_start_menu();
			do { } while (key[KEY_RIGHT]);
		}
		if (key[KEY_LEFT])
		{
			if (selecty==0)
			{
				if (full_screen==FALSE)
					full_screen=TRUE;
				else
					full_screen=FALSE;
			}
			if (selecty==1)
			{
				if (screen_height==240)
				{ screen_width=1600; screen_height=1200; }
				else if (screen_height==480)
				{ screen_width=320; screen_height=240; }
				else if (screen_height==600)
				{ screen_width=640; screen_height=480; }
				else if (screen_height==768)
				{ screen_width=800; screen_height=600; }
				else if (screen_height==1024)
				{ screen_width=1024; screen_height=768; }
				else
				{ screen_width=1280; screen_height=1024; }
			}
			if (selecty==2)
			{
				if (color_depth==16)
					color_depth=0;
				else if (color_depth==32)
					color_depth=16;
				else
					color_depth=32;
			}
			if (selecty==3)
			{
				if (brightness>0) brightness-=0.05;
			}
			show_start_menu();
			do { } while (key[KEY_LEFT]);
		}

		if (key[KEY_ESC])
			return -1;
	} while (!key[KEY_ENTER]);
	do
	{
	} while (key[KEY_ENTER]);

	fp=pack_fopen("paniccfg.dat",F_WRITE_PACKED);
	pack_fwrite(&screen_width, sizeof(int), fp);
	pack_fwrite(&screen_height, sizeof(int), fp);
	pack_fwrite(&color_depth, sizeof(int), fp);
	pack_fwrite(&full_screen, sizeof(int), fp);
	pack_fwrite(&brightness, sizeof(double), fp);
	pack_fclose(fp);
	
	return 0;
}

int program_init(void)
{
	allegro_init();
	set_window_title("Speedhack 2002");

	LOCK_FUNCTION(secs_timer);
	LOCK_FUNCTION(game_timer);
	LOCK_VARIABLE(secs);
	LOCK_VARIABLE(camera);
	LOCK_VARIABLE(xpos);
	LOCK_VARIABLE(ypos);
	LOCK_VARIABLE(zpos);
	LOCK_VARIABLE(direction);
	LOCK_VARIABLE(movement);
	LOCK_VARIABLE(nextdir);
	LOCK_VARIABLE(checkfalldown);

	install_keyboard();
	install_timer();
	srand(time(NULL));
	set_keyboard_rate(0,0);

/*
	// set graphics mode, quit when failure
	if (desktop_color_depth()!=0)
		set_color_depth(desktop_color_depth());
	else 
		set_color_depth(24);
*/
/*
	set_color_depth(16);
	if (set_gfx_mode(GFX_OPENGL_FULLSCREEN, 640, 480, 0, 0) < 0) {
		if (desktop_color_depth()!=0)
			set_color_depth(desktop_color_depth());
		else 
			set_color_depth(24);
		if (set_gfx_mode(GFX_OPENGL_WINDOWED, 640, 480, 0, 0) < 0) {
			allegro_message ("Error setting graphics mode:\n%s\n", allegro_error);
			return -1;
		}
	}
*/
	if (start_menu()!=0)
		return -1;

	init_gl();
	if (color_depth==0)
	{
		if (desktop_color_depth()!=0)
			set_color_depth(desktop_color_depth());
		else 
			set_color_depth(32);
	} else
	{
		set_color_depth(color_depth);
	}
	if (full_screen==TRUE)
	{
		if (set_gfx_mode(GFX_OPENGL_FULLSCREEN, screen_width,screen_height, 0, 0) < 0) {
			allegro_message ("Error setting graphics mode:\n%s\n", allegro_error);
			return -1;
		}
	} else
	{
		if (set_gfx_mode(GFX_OPENGL_WINDOWED, screen_width,screen_height, 0, 0) < 0) {
			allegro_message ("Error setting graphics mode:\n%s\n", allegro_error);
			return -1;
		}
	}
	allegro_gl_set_texture_format(GL_RGB8);
	install_sound();
	create_samples();
	init_circle_pos();
	allegro_gl_screen_mode (AGL_MODE_DIRECT);

	allegro_gl_begin();
	glClearColor (0.0, 0.0, 0.05, 1.0);
	glShadeModel (GL_FLAT);
	glPolygonMode (GL_FRONT_AND_BACK, GL_FILL);
	glEnable (GL_DEPTH_TEST);
	glCullFace (GL_BACK);
	glEnable (GL_CULL_FACE);
	init_fog();
	allegro_gl_end();

	return 0;
}

void secs_timer(void) 
{ 
	secs++; 
} 
END_OF_FUNCTION(secs_timer);
