#ifndef MAP_H
#define MAP_H

#include <math.h>
#include <allegro.h>

#include "alleggl.h"
#include "triangles.h"

typedef struct SMALLMAPITEM
{
	int tile_id;
	double height;

	struct SMALLMAPITEM *next;
} SMALLMAPITEM;

typedef struct OBJECTINFO
{
	double x,y,z;
	struct TRIANGLEINFO *objectdata;
	int object_id;
	int size;
	double scalefac;
	double rotate;

	struct OBJECTINFO *nextobject;
} OBJECTINFO;

typedef struct TILEPOS
{
	double x,y,z;
	struct TRIANGLEINFO *tiledata;
	int tile_id;

	struct TILEPOS *nexttile;
} TILEPOS;

typedef struct MAPPOS
{
	struct TILEPOS *tilelist;
} MAPPOS;

typedef struct TILECOOLDOWN
{
	int timer, initialtimer;
	struct TILEPOS *tileinfo;

	struct TILECOOLDOWN *next;
} TILECOOLDOWN;

extern TILECOOLDOWN			*cooldown;
extern struct MAPPOS		mapdata[20][20];
extern struct OBJECTINFO	*objectinfo;
extern struct SMALLMAPITEM	*smallmap[200][200];

extern struct OBJECTINFO	*shadow_object;

void draw_objects(void);
void draw_tiles(double x, double y);
double fall_down(double x, double y, double z, TILEPOS **tileinfo);
void add_tilecooldown(int timer, TILEPOS *tileinfo);
void check_cooldown(void);
void clear_map(void);
void free_map(void);
void add_tile(double x, double y, double z, struct TRIANGLEINFO *tiledata, int tile_id);
void add_object(double x, double y, double z, TRIANGLEINFO *objectdata, int object_id, int size);
void free_objects(void);
OBJECTINFO *intersects_with(double x, double y, double z);
int direction_possible(double x, double y, double z);
int direction_falldown(double x, double y, double z);
int direction_falldead(double x, double y, double z);

#endif
