#include <math.h>
#include <allegro.h>
#include "alleggl.h"
#include "sound.h"
#include "map.h"
#include "drawing.h"
#include "init.h"
#include "circle.h"

typedef struct MONSTERINFO
{
	int direction; 
	int facedir;
	double movement;
	double fallspeed;
	struct OBJECTINFO	*monster_object;
} MONSTERINFO;

/* ********** variables ********** */

double					xpos,ypos,zpos;
double					speedx,speedy,fallspeed;
int						direction,nextdir;
int						checkfalldown;
double					movement;
MONSTERINFO				monster[32];
int						nrmonsters;
int						endgame;
int						foodcount,score,prevscore;
int						enableplayer;
int						skip;
int						showmap;
int						waitkey;
int						lives;

/* ********** procedure declarations ********** */

void game_timer(void);
void set_camera_position(void);
void keyboard(void);
void display(void);
void restart(void);
void restart_monsters(void);

/* ********** procedure implementations ********** */

void increase_score(int value)
{
	score+=value;
	prevscore+=value;
	if (prevscore>=200)
	{
		prevscore-=200;
		lives++;
	}
}

void game_movement(void)
{
	int monstercount,newdir,olddir,i;
	double x,y,z;
	int bleft,bright,bup,bdown;
	TILEPOS *temptile;

	if (waitkey>0) waitkey--;
	if (key[KEY_PGUP] && camera.xangle>-30) camera.xangle--;
	if (key[KEY_PGDN] && camera.xangle<0) camera.xangle++;

	if (enableplayer)
	{
		if (movement>0 && direction!=0)
		{
			if (direction==1) ypos+=0.5;
			if (direction==2) xpos+=0.5;
			if (direction==3) ypos-=0.5;
			if (direction==4) xpos-=0.5;
			movement-=0.5;
		}

		if (movement<=0)
		{
			direction=nextdir;
			nextdir=0;
		
			if (direction==1 && !direction_possible(xpos,ypos+5,zpos)) direction=0;
			if (direction==2 && !direction_possible(xpos+5,ypos,zpos)) direction=0;
			if (direction==3 && !direction_possible(xpos,ypos-5,zpos)) direction=0;
			if (direction==4 && !direction_possible(xpos-5,ypos,zpos)) direction=0;

			if (direction!=0)
				movement=5;
		}
	}

	// move the monsters
	for (monstercount=0; monstercount<nrmonsters; monstercount++)
	{
		x=monster[monstercount].monster_object->x;
		y=monster[monstercount].monster_object->y;
		z=monster[monstercount].monster_object->z;

		// monster catches player
		if (enableplayer &&
			x>=xpos-2 && x<=xpos+2 &&
			y>=ypos-2 && y<=ypos+2 &&
			z>=zpos && z<=zpos+4)
		{
			lives--;
			if (lives<0)
			{
				endgame=TRUE;
			} else
			{
				restart();
				restart_monsters();
				x=monster[monstercount].monster_object->x;
				y=monster[monstercount].monster_object->y;
				z=monster[monstercount].monster_object->z;
			}
			play_sample(10,255,127,1000);
		}
		
		if (monster[monstercount].movement>0 && monster[monstercount].direction!=0)
		{
			monster[monstercount].monster_object->rotate=((6-monster[monstercount].direction)*90+30);
			if ((monstercount % 5)==4)
			{
				if (monster[monstercount].direction==1) y+=0.25;
				if (monster[monstercount].direction==2) x+=0.25;
				if (monster[monstercount].direction==3) y-=0.25;
				if (monster[monstercount].direction==4) x-=0.25;
			} else
			{
				if (monster[monstercount].direction==1) y+=0.25;
				if (monster[monstercount].direction==2) x+=0.25;
				if (monster[monstercount].direction==3) y-=0.25;
				if (monster[monstercount].direction==4) x-=0.25;
			}
			monster[monstercount].movement-=0.25;
		}
		if (monster[monstercount].movement<=0)
		{
			//olddir=monster[monstercount].direction;
			olddir=monster[monstercount].facedir;
			newdir=0;

			bleft=FALSE; bright=FALSE; bup=FALSE; bdown=FALSE;
			if (direction_possible(x-5,y,z) && !direction_falldead(x-5,y,z)) bleft=TRUE;
			if (direction_possible(x+5,y,z) && !direction_falldead(x+5,y,z)) bright=TRUE;
			if (direction_possible(x,y-5,z) && !direction_falldead(x,y-5,z)) bdown=TRUE;
			if (direction_possible(x,y+5,z) && !direction_falldead(x,y+5,z)) bup=TRUE;

			if (bleft && olddir==4) newdir=4;
			if (bright && olddir==2) newdir=2;
			if (bup && olddir==1) newdir=1;
			if (bdown && olddir==3) newdir=3;

			if (rand()%100<=50 || olddir!=newdir  || monstercount<=nrmonsters/2)
			{
				// try to catch the player
				if (rand()%2==0)
				{
					if (bleft && x>xpos && olddir!=2) newdir=4;
					if (bright && x<xpos && olddir!=4) newdir=2;
					if (bup && y<ypos && olddir!=3) newdir=1;
					if (bdown && y>ypos && olddir!=1) newdir=3;
				} else
				{
					if (bdown && y>ypos && olddir!=1) newdir=3;
					if (bup && y<ypos && olddir!=3) newdir=1;
					if (bright && x<xpos && olddir!=4) newdir=2;
					if (bleft && x>xpos && olddir!=2) newdir=4;
				}
			}

			if (rand()%100<=25 || olddir!=newdir || monstercount<=nrmonsters/2)
			{
				if (olddir==4 && x<xpos && bup) newdir=1;
				if (olddir==4 && x<xpos && bdown) newdir=3;
				if (olddir==4 && x<xpos && bup && bdown) 
				{	if (y<ypos) newdir=1; else newdir=3;
				}
				if (olddir==2 && x>xpos && bdown) newdir=3;
				if (olddir==2 && x>xpos && bup) newdir=1;
				if (olddir==2 && x>xpos && bdown && bup) 
				{	if (y<ypos) newdir=1; else newdir=3;
				}
				if (olddir==1 && y>ypos && bleft) newdir=4;
				if (olddir==1 && y>ypos && bright) newdir=2;
				if (olddir==1 && y>ypos && bleft && bright) 
				{	if (x<xpos) newdir=2; else newdir=4;
				}
				if (olddir==3 && y<ypos && bright) newdir=2;
				if (olddir==3 && y<ypos && bleft) newdir=4;
				if (olddir==3 && y<ypos && bleft && bright)
				{	if (x<xpos) newdir=2; else newdir=4;
				}
			}

			if (bleft && newdir==0 && olddir!=2) newdir=4;
			if (bright && newdir==0 && olddir!=4) newdir=2;
			if (bup && newdir==0 && olddir!=3) newdir=1;
			if (bdown && newdir==0 && olddir!=1) newdir=3;

			if (monster[monstercount].fallspeed>0)
				newdir=0;
			if (fall_down(x,y,z,&temptile) && (rand()%100)<=5) // monsters love trampolines
				newdir=0;
			if (temptile->tile_id==20 && (rand()%100)<=95 && z<zpos)
				newdir=0;

			for (i=0; i<monstercount; i++)
			{
				if (x>=monster[i].monster_object->x-3 &&
					y>=monster[i].monster_object->y-3 &&
					z>=monster[i].monster_object->z-3 &&
					x<=monster[i].monster_object->x+3 &&
					y<=monster[i].monster_object->y+3 &&
					z<=monster[i].monster_object->z+3)
					newdir=0;
			}

			monster[monstercount].direction=newdir;
			if (newdir!=0)
				monster[monstercount].movement=5;

			if (monster[monstercount].direction!=0)
				monster[monstercount].facedir=monster[monstercount].direction;
		}
		monster[monstercount].monster_object->x=x;
		monster[monstercount].monster_object->y=y;
		monster[monstercount].monster_object->z=z;
	}

	camera.translatex = (camera.translatex*9 + xpos)/10;
	camera.translatey = (camera.translatey*9 + -ypos)/10;
	camera.translatez = (camera.translatez*9 + (zpos-3))/10;

}

void game_timer(void)
{
	checkfalldown++;

} END_OF_FUNCTION(game_timer);

void keyboard(void)
{
	if (key[KEY_ESC]) endgame=TRUE;
	if (key[KEY_TAB] && waitkey==0) { showmap=1-showmap; waitkey=15; }

	if (movement<=1 || (movement<=0 && keyboard_action==1))
	{
		if (keyboard_action==1)
			nextdir=0;
		if (key[KEY_LEFT]) nextdir=4;
		if (key[KEY_RIGHT]) nextdir=2;
		if (key[KEY_UP]) nextdir=1;
		if (key[KEY_DOWN]) nextdir=3;

	}

	if (key[KEY_A] && waitkey==0)
	{
		play_sample(11, 255, 127, 1000);
		waitkey=15;
	}

	rotatebody++;
	if (rotatebody>=360) 
		rotatebody-=360;

	set_camera_position();

	display();
}

void display(void)
{

	struct OBJECTINFO		*currobject;
	double					x,y;
	int						i,tempscore;

	allegro_gl_begin();

    // Clear the RGB buffer and the depth buffer
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	if (enableplayer)
	{
		shadow_object->x=xpos;
		shadow_object->y=ypos;
		shadow_object->scalefac=(1/(1+(zpos-shadow_object->z)/25));
		draw_abombman(xpos,ypos,zpos+3.6);

	}
	draw_tiles(xpos,ypos);
	draw_objects();

	// minimap (toggle with tab key)
	if (enableplayer && showmap==1)
	{
		glPushMatrix();

		glTranslatef (camera.translatex, -camera.translatey, camera.translatez);
		glRotatef(-camera.zangle, 0, 0, 1);
		glRotatef(-camera.xangle, 1, 0, 0);
		glTranslatef (-camera.translatex, camera.translatey, -camera.translatez);
		glTranslatef(0, 6, 0); // move the front display further up

		tempscore=score*10;
		for (i=0; i<6; i++)
		{
			draw_object(camera.translatex+3-i,-camera.translatey-3,camera.translatez+25,numbers[tempscore % 10],1,0);
			tempscore/=10;
		}

		// lives
		for (i=0; i<lives-1; i++)
			draw_object(camera.translatex+7.5-i,-camera.translatey-0.5,camera.translatez+25,abombman_body,0.2,-1);

		// player
		x=xpos/30;
		y=ypos/30;
		draw_object(camera.translatex-5.5+x,-camera.translatey-2.25+y,camera.translatez+25,dots[1],1,0);
		currobject=objectinfo;
		while (currobject!=NULL)
		{
			if (currobject->object_id==10)
			{
				// monster
				x=(currobject->x)/30;
				y=(currobject->y)/30;
				draw_object(camera.translatex-5.5+x,-camera.translatey-2.25+y,camera.translatez+25,dots[0],1,0);
			}
			if (currobject->object_id==1)
			{
				// food
				x=(currobject->x)/30;
				y=(currobject->y)/30;
				draw_object(camera.translatex-5.5+x,-camera.translatey-2.25+y,camera.translatez+25,dots[2],1,0);
			}
			currobject=currobject->nextobject;
		}
	
		glPopMatrix();
	}

	glFlush();

	allegro_gl_flip();
	allegro_gl_end();

	frames++;
}

void place_objects(void)
{
	int i,i1,i2;
	double y;
	TILEPOS *tileinfo;

	// food
	foodcount=0;
	for (i1=-60; i1<=60; i1+=5)
		for (i2=-60; i2<=60; i2+=5)
			if ((((i1+500) + ((i2+500) % 2)) % 2)==1)
		{
			y=fall_down(i1,i2,100,&tileinfo);
				
			//if (!direction_falldead(i1,i2,1))
			if (y>-5 && y<15 && i1!=55 && i2!=55 && i1!=-55 && i2!=-55 && tileinfo->tile_id!=20)
			{
				add_object(i1,i2,y+1,food,1,1);
				foodcount++;
			}
		}

	// shadow
	add_object(xpos,ypos,zpos,shadow,0,1); shadow_object=objectinfo;

	// monsters
	i1=-50;
	i2=-50;
	for (i=0; i<nrmonsters; i++)
	{
		add_object(i1,i2,200+i*20,ghost,10,3); 
		monster[i].monster_object=objectinfo;
		monster[i].direction=0;
		monster[i].movement=0;
		monster[i].fallspeed=0;
		i1+=100;
		if (i1>50)
		{
			i1=-50;
			i2+=100;
			if (i2>50)
				i2=-50;
		}
	}
}

void restart_monsters(void)
{
	int i,i1,i2;

	i1=-50;
	i2=-50;
	for (i=0; i<nrmonsters; i++)
	{
		monster[i].monster_object->x=i1;
		monster[i].monster_object->y=i2;
		monster[i].monster_object->z=200+i*20;
		monster[i].direction=0;
		monster[i].movement=0;
		monster[i].fallspeed=0;
		i1+=100;
		if (i1>50)
		{
			i1=-50;
			i2+=100;
			if (i2>50)
				i2=-50;
		}
	}
}

void create_peace(void)
{
	int i1,i2;
	int peace[15][15] = {
		{ 0,0,0,0,0,2,1,1,1,2,0,0,0,0,0 },
		{ 0,0,0,2,1,1,3,1,3,1,1,2,0,0,0 },
		{ 0,0,2,1,3,3,3,1,3,3,3,1,2,0,0 },
		{ 0,2,1,3,3,3,3,1,3,3,3,3,1,2,0 },
		{ 0,1,3,3,3,3,3,1,3,3,3,3,3,1,0 },
		{ 2,1,3,3,3,3,3,1,3,3,3,3,3,1,2 },
		{ 1,3,3,3,3,3,2,1,2,3,3,3,3,3,1 },
		{ 1,3,3,3,3,2,1,1,1,2,3,3,3,3,1 },
		{ 1,3,3,3,2,1,3,1,3,1,2,3,3,3,1 },
		{ 2,1,3,2,1,3,3,1,3,3,1,2,3,1,2 },
		{ 0,1,2,1,3,3,3,1,3,3,3,1,2,1,0 },
		{ 0,2,1,3,3,3,3,1,3,3,3,3,1,2,0 },
		{ 0,0,2,1,3,3,3,1,3,3,3,1,2,0,0 },
		{ 0,0,0,2,1,1,3,1,3,1,1,2,0,0,0 },
		{ 0,0,0,0,0,2,1,1,1,2,0,0,0,0,0 },
	};

	for (i1=0; i1<15; i1++)
		for (i2=0; i2<15; i2++)
			if (peace[i1][i2]!=0)
		{
			if (peace[i1][i2]==1)
				add_tile(-130+i2*5,20-i1*5,45,floor_tile[2],2);
			if (peace[i1][i2]==2)
				add_tile(-130+i2*5,20-i1*5,45,floor_tile[3],2);
		}
}

void create_map(void)
{
	int i,i1,i2,tile1;

	for (i=-60; i<=60; i+=5)
	{
		if ((i+1000)%2==1)
		{
			add_tile(i,-65,3,floor_tile[rand()%2],1);
			add_tile(i,65,3,floor_tile[rand()%2],1);
			add_tile(-65,i,3,floor_tile[rand()%2],1);
			add_tile(65,i,3,floor_tile[rand()%2],1);
		} else
		{
			add_tile(i,-65,5,floor_tile[rand()%2],1);
			add_tile(i,65,5,floor_tile[rand()%2],1);
			add_tile(-65,i,5,floor_tile[rand()%2],1);
			add_tile(65,i,5,floor_tile[rand()%2],1);
		}
	}
	add_tile(-65,-65,8,floor_tile[rand()%2],1);
	add_tile(-65,65,8,floor_tile[rand()%2],1);
	add_tile(65,-65,8,floor_tile[rand()%2],1);
	add_tile(65,65,8,floor_tile[rand()%2],1);

	for (i=-55; i<=55; i+=5)
	{
		//add_tile(i,15,-10,floor_tile[rand()%2],1);
		//add_tile(i,-15,-10,floor_tile[rand()%2],1);
		add_tile(15,i,-10,floor_tile[rand()%2],1);
		add_tile(-15,i,-10,floor_tile[rand()%2],1);
	}
//	add_tile(-55,15,-13,trampoline[0],20);
//	add_tile(55,15,-13,trampoline[0],20);
//	add_tile(-55,-15,-13,trampoline[0],20);
//	add_tile(55,-15,-13,trampoline[0],20);
	add_tile(20,55,-13,trampoline[0],20);
	add_tile(20,-55,-13,trampoline[0],20);
	add_tile(-20,55,-13,trampoline[0],20);
	add_tile(-20,-55,-13,trampoline[0],20);

	add_tile(15,55,0,floor_tile[4+rand()%2],2);
	add_tile(15,-55,0,floor_tile[4+rand()%2],2);
	add_tile(-15,55,0,floor_tile[4+rand()%2],2);
	add_tile(-15,-55,0,floor_tile[4+rand()%2],2);
	for (i=-45; i<=-25; i+=5)
	{
		add_tile(i,55,4,floor_tile[rand()%2],1);
		add_tile(i,-55,4,floor_tile[rand()%2],1);
	}
	for (i=-10; i<=-5; i+=5)
	{
		add_tile(i,55,4,floor_tile[rand()%2],1);
		add_tile(i,-55,4,floor_tile[rand()%2],1);
	}
	for (i=25; i<=45; i+=5)
	{
		add_tile(i,55,4,floor_tile[rand()%2],1);
		add_tile(i,-55,4,floor_tile[rand()%2],1);
	}
	for (i=5; i<=10; i+=5)
	{
		add_tile(i,55,4,floor_tile[rand()%2],1);
		add_tile(i,-55,4,floor_tile[rand()%2],1);
	}

	add_tile(-10,-30,0,floor_tile[4+rand()%2],2);

	add_tile(15,5,0,floor_tile[4+rand()%2],2);
	add_tile(15,-5,0,floor_tile[4+rand()%2],2);
	add_tile(-15,5,0,floor_tile[4+rand()%2],2);
	add_tile(-15,-5,0,floor_tile[4+rand()%2],2);
	add_tile(20,5,0,floor_tile[4+rand()%2],2);
	add_tile(20,-5,0,floor_tile[4+rand()%2],2);
	add_tile(-20,5,0,floor_tile[4+rand()%2],2);
	add_tile(-20,-5,0,floor_tile[4+rand()%2],2);

	for (i=-60; i<=60; i+=5)
	{
		if (i==15 || i==-15) tile1=1; else tile1=0;
		add_tile(i,0,0,floor_tile[tile1*4+rand()%2],1+tile1);
		tile1=0;
		add_tile(i,-60,0,floor_tile[tile1*4+rand()%2],1+tile1);
		add_tile(i,60,0,floor_tile[tile1*4+rand()%2],1+tile1);
		add_tile(-60,i,0,floor_tile[tile1*4+rand()%2],1+tile1);
		add_tile(60,i,0,floor_tile[tile1*4+rand()%2],1+tile1);
		add_tile(0,i,0,floor_tile[tile1*4+rand()%2],1+tile1);

		if (i==15 || i==-15) tile1=1; else tile1=0;
		add_tile(i,-50,0,floor_tile[tile1*4+rand()%2],1+tile1);
		add_tile(i,50,0,floor_tile[tile1*4+rand()%2],1+tile1);
		tile1=0;
		add_tile(-50,i,0,floor_tile[tile1*4+rand()%2],1+tile1);
		add_tile(50,i,0,floor_tile[tile1*4+rand()%2],1+tile1);

	}

	for (i1=-20; i1<=20; i1+=5)
		for (i2=-20; i2<=20; i2+=5)
			if ((i1<-5 || i1>5) && (i2<-5 || i2>5))
				if (i1 !=-15 || i2 != 15)
				{
					//if (i1==15 || i1==-15 || i2==15 || i2==-15) 
					if (i1==15 || i1==-15) 
					{
						add_tile(i1,i2,10,floor_tile[6+rand()%2],30);
					} else
					{
						add_tile(i1,i2,10,floor_tile[rand()%2],1);
					}
				}

	for (i1=-45; i1<=-35; i1+=5)
		for (i2=-45; i2<=-35; i2+=5)
			add_tile(i1,i2,5,floor_tile[rand()%2],1);
	for (i1=35; i1<=45; i1+=5)
		for (i2=-45; i2<=-35; i2+=5)
			add_tile(i1,i2,5,floor_tile[rand()%2],1);
	for (i1=-45; i1<=-35; i1+=5)
		for (i2=35; i2<=45; i2+=5)
			add_tile(i1,i2,5,floor_tile[rand()%2],1);
	for (i1=35; i1<=45; i1+=5)
		for (i2=35; i2<=45; i2+=5)
			add_tile(i1,i2,5,floor_tile[rand()%2],1);

	add_tile(5,10,-3,trampoline[0],20);
	add_tile(-10,5,-3,trampoline[0],20);
	add_tile(10,-5,-3,trampoline[0],20);
	add_tile(-15,15,9-3,trampoline[0],20);

	add_tile(-55,-55,-3,trampoline[0],20);
	add_tile(55,-55,-3,trampoline[0],20);
	add_tile(-55,55,-3,trampoline[0],20);
	add_tile(55,55,-3,trampoline[0],20);

	add_tile(-30,40,-3,trampoline[0],20);
	add_tile(30,40,-3,trampoline[0],20);
	add_tile(-30,-40,-3,trampoline[0],20);
	add_tile(30,-40,-3,trampoline[0],20);

	add_tile(-25,15,-3,trampoline[0],20);
	add_tile(-25,-15,-3,trampoline[0],20);
	add_tile(25,15,-3,trampoline[0],20);
	add_tile(25,-15,-3,trampoline[0],20);

	for (i=-45; i<=-25; i+=5)
	{
		add_tile(i,10,0,bridge[0],3);
		add_tile(i,-10,0,bridge[0],3);
		add_tile(i,20,0,bridge[0],3);
		add_tile(i,-20,0,bridge[0],3);
	}
	for (i=25; i<=45; i+=5)
	{
		add_tile(i,10,0,bridge[0],3);
		add_tile(i,-10,0,bridge[0],3);
		add_tile(i,20,0,bridge[0],3);
		add_tile(i,-20,0,bridge[0],3);
	}
	for (i=-5; i<=5; i+=5)
	{
		add_tile(i,15,10,bridge[0],3);
		add_tile(i,-15,10,bridge[0],3);
	}
	for (i=-25; i<=25; i+=5)
		if (i!=0)
		{
			add_tile(i,40,0,bridge[0],3);
			add_tile(i,-40,0,bridge[0],3);
		}

	place_objects();
}

void hit_tile(TILEPOS *tileinfo, double fallto, double *zpos, double *fallspeed, int wastetrampoline, 
			  double x, double y, double z)
{
	int i;
	double dist, sounddirection;

	switch (tileinfo->tile_id) 
	{
	case 1: 
	case 2:
	case 3:
	case 4:
	case 30:
			// solid tile
			*zpos=fallto; // fall exactly on the tile
			*fallspeed=0; // stop falling
		break;
	case 20: // trampoline
			if (tileinfo->tiledata==NULL)
			{
				// keep on falling, the trampoline was red and was removed
			}
			else
			{
				*zpos=fallto; // fall exactly on the tile
				*fallspeed=-*fallspeed*1.35; // reverse falling
				if (*fallspeed<-1) *fallspeed=-1;

				if (wastetrampoline==TRUE)
				{
					play_sample(1,255,127,1000+(int)(*fallspeed*-200));

					// if the trampoline was red, remove it
					if (tileinfo->tiledata==trampoline[3])
						tileinfo->tiledata=NULL;

					// change the trampoline, from white->yellow->orange->red
					for (i=2; i>=0; i--)
						if (tileinfo->tiledata==trampoline[i])
							tileinfo->tiledata=trampoline[i+1];

					add_tilecooldown(200,tileinfo);
				} else
				{
					// monster on trampoline
					dist=sqrt((x-xpos)*(x-xpos)+(y-ypos)*(y-ypos))*5;
					dist=255-dist;
					if (dist<0) dist=0;
					sounddirection=(x-xpos)*20;
					if (sounddirection<0) sounddirection=0;
					if (sounddirection>255) sounddirection=255;
					play_sample(1,(int)dist,(int)sounddirection,1000+(int)(*fallspeed*-200));
				}
			}

		break;
	default:  // ignore tile
		break;
	}

}

void check_falldown(void)
{
	double fallto;
	TILEPOS *tileinfo;
	int monstercount;

	if (enableplayer)
	{
		// tileinfo will be a pointer to the tile under the player
		tileinfo=NULL;
		fallto=fall_down(xpos,ypos,zpos,&tileinfo);
		shadow_object->z=fallto+0.05;
		zpos-=fallspeed;
		if (zpos<=fallto)
		{
			if (tileinfo!=NULL)
			{
				// the player reached the tile under him
				hit_tile(tileinfo, fallto, &zpos, &fallspeed, TRUE, xpos, ypos, zpos);
			}
			else
			{
				// nothing under the player, but don't continue to fall, stop at -20
				zpos=fallto;
				fallspeed=0;
				play_sample(10,255,127,1000);
				lives--;
				if (lives<=0)
				{
					endgame=TRUE;
				} else
				{
					restart();
					restart_monsters();
				}
			}
		}
		if (zpos>fallto)
		{
			fallspeed+=0.02;
			if (fallspeed>1) fallspeed=1;
		}
	}

	// and now for the monsters
	for (monstercount=0; monstercount<nrmonsters; monstercount++)
	{
		tileinfo=NULL;
		fallto=fall_down(monster[monstercount].monster_object->x,
			monster[monstercount].monster_object->y,
			monster[monstercount].monster_object->z,&tileinfo);
		monster[monstercount].monster_object->z-=monster[monstercount].fallspeed;
		if (monster[monstercount].monster_object->z<=fallto)
		{
			if (tileinfo!=NULL)
			{
				// the player reached the tile under him
				hit_tile(tileinfo, fallto, &monster[monstercount].monster_object->z, 
					&monster[monstercount].fallspeed, FALSE,
					monster[monstercount].monster_object->x,
					monster[monstercount].monster_object->y,
					monster[monstercount].monster_object->z);
			}
			else
			{
				// nothing under the monster, but don't continue to fall, stop at -20
				increase_score(10);
				monster[monstercount].monster_object->z=fallto;
				monster[monstercount].fallspeed=0;
				monster[monstercount].monster_object->z=500;
				monster[monstercount].monster_object->x=0;
				monster[monstercount].monster_object->y=0;
			}
		}
		if (monster[monstercount].monster_object->z>fallto)
		{
			monster[monstercount].fallspeed+=0.02;
			if (monster[monstercount].fallspeed>1) monster[monstercount].fallspeed=1;
		}
	}
}

void check_intersect(void)
{
	OBJECTINFO *intersect;

	intersect=intersects_with(xpos,ypos,zpos);
	if (intersect!=NULL)
	{
		switch (intersect->object_id)
		{
		case 1: // food
			intersect->object_id=-1;
			intersect->objectdata=NULL;
			foodcount--;
			increase_score(1);
			if (foodcount<=0)
			{
				free_objects();
				zpos+=5;
				fallspeed=-0.5; // jump of joy, you have all the food
				if (nrmonsters<30)
					nrmonsters++;
				place_objects();
				//endgame=TRUE;
			}
			play_sample(5,255,127,900+(rand()%300));
			break;
		}
	}
}

void restart(void)
{
	xpos=0; ypos=0; zpos=100; direction=0; movement=0; nextdir=0; 
	fallspeed=0; checkfalldown=0;
	speedx=0; speedy=0;
}

void start_settings(void)
{
	restart();
	camera.translatez=zpos; camera.translatex=0; camera.translatey=0;
	score=0; prevscore=0;
	endgame=FALSE;
	nrmonsters=2;
	enableplayer=TRUE;
	lives=3;

	clear_map();
	create_map();
}

void intro_screen(void)
{
	int i;

		create_peace();
		for (i=0; i<8; i++)
		{
			add_object(-95,-15,200+i*50,ghost,10,3); 
			monster[i].monster_object=objectinfo;
			monster[i].direction=0;
			monster[i].movement=0;
			monster[i].fallspeed=0;
		}
		nrmonsters=8;

		xpos=-95;
		ypos=-15;
		zpos=50;
		i=0;
		enableplayer=FALSE;
		do
		{
			if (checkfalldown>0)
			{
				// stuff i dont dare to put in a timer
				checkfalldown--;

				i+=10;
				if (i>=5760)
					i-=5760;
				xpos=-95+get_x(i,20);
				ypos=-15+get_y(i,20);
				game_movement();
				check_falldown();
				check_cooldown();
				check_intersect();
			}
		
			keyboard();
		} while (!key[KEY_ESC] && !key[KEY_SPACE] && !key[KEY_ENTER]);

		free_map();
		free_objects();
}

int main()
{
	if (program_init() != 0)
		return -1;

	camera.xangle=-15; camera.zangle=5; camera.yangle=0;
	camera.dist=35;
	skip=1; showmap=1; waitkey=0;
	create_trianglelists();
	cooldown=NULL;
	objectinfo=NULL;
	install_int (secs_timer, 1000);
	install_int (game_timer, 20);
	
	load_music();
	start_music();

	do
	{
		start_settings();

		if (skip==0)
		{
			do 
			{
				if (checkfalldown>0)
				{
					// stuff i dont dare to put in a timer
					checkfalldown--;

					game_movement();
					check_falldown();
					check_cooldown();
					check_intersect();

				}
		
				keyboard();
			} while (!endgame);
	
			do
			{ 
			} while (key[KEY_ESC]);

		}
		skip=0;
		free_map();
		free_objects();
		intro_screen();

	} while (!key[KEY_ESC]);

    stop_music();
	destroy_music();
	
	program_exit();	
	return 0;
}
END_OF_MAIN();



