#include <allegro.h>
#include <math.h>
#include "alleggl.h"
#include "utils.h"
#include "triangles.h"
#include "circle.h"

int						model_detail=4; // 0 = slow comps..very high for very fast comps
double					brightness=0.25;

TRIANGLEINFO			*abombman_body;
TRIANGLEINFO			*floor_tile[8];
TRIANGLEINFO			*trampoline[4];
TRIANGLEINFO			*food;
TRIANGLEINFO			*bridge[2];
TRIANGLEINFO			*shadow;
TRIANGLEINFO			*ghost;
TRIANGLEINFO			*dots[3];
TRIANGLEINFO			*numbers[10];

TRIANGLEINFO *add_triangle(TRIANGLEINFO *trianglelist, double x[3], double y[3], double z[3], double scalefac);
void free_trianglelist(TRIANGLEINFO *trianglelist);
void trianglelist_to_vertex(TRIANGLEINFO *trianglelist);
void create_trianglelists(void);
void destroy_trianglelists(void);
TRIANGLEINFO *set_trianglecolor(TRIANGLEINFO *trianglelist, float red, float green, float blue, float alpha);

TRIANGLEINFO *create_abombman_body(double scalefac);
TRIANGLEINFO *create_floor_tile(double scalefac, double brightness, double depth);
TRIANGLEINFO *create_trampoline(double scalefac, double depth, double color1[3], double color2[3]);
TRIANGLEINFO *create_food(double scalefac);
TRIANGLEINFO *create_bridge(double scalefac);
TRIANGLEINFO *create_shadow(double scalefac);
TRIANGLEINFO *create_monster(double scalefac);
TRIANGLEINFO *create_dot(double scalefac, double color[3]);
TRIANGLEINFO *create_numbers(double scalefac, char *number);

void create_trianglelists(void);
void destroy_trianglelists(void);


TRIANGLEINFO *add_triangle(TRIANGLEINFO *trianglelist, double x[3], double y[3], double z[3], double scalefac)
{
	TRIANGLEINFO *newtriangle;
	int i;
	double d1[3], d2[3];

	// reserve memory
	newtriangle=(TRIANGLEINFO*)malloc(sizeof(TRIANGLEINFO));
	// initialize pointers
	newtriangle->nexttriangle=NULL;
	newtriangle->color=NULL;
	// apply scalefac and store x,y,z coordinates
	for (i=0; i<3; i++)
	{
		newtriangle->x[i]=x[i]*scalefac;
		newtriangle->y[i]=y[i]*scalefac;
		newtriangle->z[i]=z[i]*scalefac;
	}
	// calculate norm
	d1[0]=x[1]-x[0];
	d1[1]=y[1]-y[0];
	d1[2]=z[1]-z[0];
	d2[0]=x[2]-x[0];
	d2[1]=y[2]-y[0];
	d2[2]=z[2]-z[0];
	normcrossprod(d1,d2,newtriangle->n);
	// add new triangle to the list
	if (trianglelist!=NULL)
		newtriangle->nexttriangle=trianglelist;
	trianglelist=newtriangle;

	return trianglelist;
}

TRIANGLEINFO *set_trianglecolor(TRIANGLEINFO *trianglelist, float red, float green, float blue, float alpha)
{
	TRIANGLECOLOR *newcolor;

	red=(red+brightness)/(1+brightness);
	green=(green+brightness)/(1+brightness);
	blue=(blue+brightness)/(1+brightness);

	newcolor=(TRIANGLECOLOR*)malloc(sizeof(TRIANGLECOLOR));
	newcolor->red=red;
	newcolor->green=green;
	newcolor->blue=blue;
	newcolor->alpha=alpha;
	if (trianglelist->color!=NULL)
		free(trianglelist->color);
	trianglelist->color=newcolor;

	return trianglelist;
}

void free_trianglelist(TRIANGLEINFO *trianglelist)
{
	TRIANGLEINFO *currtriangle, *deletetriangle;

	currtriangle=trianglelist;
	while (currtriangle!=NULL)
	{
		deletetriangle=currtriangle;		
		currtriangle=currtriangle->nexttriangle;
		if (deletetriangle->color!=NULL)
			free(deletetriangle->color);
		free(deletetriangle);
	}
}

void trianglelist_to_vertex(TRIANGLEINFO *trianglelist)
{
	TRIANGLEINFO *currtriangle;

	currtriangle=trianglelist;
	while (currtriangle!=NULL)
	{
		if (currtriangle->color!=NULL)
			//glColor3f(currtriangle->color->red, currtriangle->color->green, currtriangle->color->blue);
			glColor4f(currtriangle->color->red, currtriangle->color->green, currtriangle->color->blue, currtriangle->color->alpha);
		glNormal3dv(currtriangle->n);
		glVertex3d(currtriangle->x[0], currtriangle->y[0], currtriangle->z[0]);
		glVertex3d(currtriangle->x[1], currtriangle->y[1], currtriangle->z[1]);
		glVertex3d(currtriangle->x[2], currtriangle->y[2], currtriangle->z[2]);
		currtriangle=currtriangle->nexttriangle;
	}
}

TRIANGLEINFO *create_food(double scalefac)
{
	TRIANGLEINFO *food;
	double vx[3], vy[3], vz[3];

	food=NULL;
	vx[0]=-1; vy[0]=1; vz[0]=0;
	vx[1]=-1; vy[1]=-1; vz[1]=0;
	vx[2]=0; vy[2]=0; vz[2]=1;
	food=add_triangle(food,vx,vy,vz,scalefac);
	food=set_trianglecolor(food,1,1,0,1);
	vx[0]=-1; vy[0]=-1; vz[0]=0;
	vx[1]=1; vy[1]=-1; vz[1]=0;
	vx[2]=0; vy[2]=0; vz[2]=1;
	food=add_triangle(food,vx,vy,vz,scalefac);
	food=set_trianglecolor(food,0.9,0.9,0,1);
	vx[0]=1; vy[0]=-1; vz[0]=0;
	vx[1]=1; vy[1]=1; vz[1]=0;
	vx[2]=0; vy[2]=0; vz[2]=1;
	food=add_triangle(food,vx,vy,vz,scalefac);
	food=set_trianglecolor(food,8,8,0,1);
	vx[0]=1; vy[0]=1; vz[0]=0;
	vx[1]=-1; vy[1]=1; vz[1]=0;
	vx[2]=0; vy[2]=0; vz[2]=1;
	food=add_triangle(food,vx,vy,vz,scalefac);
	food=set_trianglecolor(food,0.7,0.7,0,1);

	return food;
}

TRIANGLEINFO *create_bridge(double scalefac)
{
	TRIANGLEINFO *bridge;
	double vx[3], vy[3], vz[3];

	bridge=NULL;
	// 1e
	vx[0]=-4.5; vy[0]=4; vz[0]=-0.2;
	vx[1]=-4.5; vy[1]=-4; vz[1]=-0.2;
	vx[2]=-4; vy[2]=-4; vz[2]=0;
	bridge=add_triangle(bridge,vx,vy,vz,scalefac);
	vx[0]=-4; vy[0]=4; vz[0]=0;
	vx[1]=-4.5; vy[1]=4; vz[1]=-0.2;
	vx[2]=-4; vy[2]=-4; vz[2]=0;
	bridge=add_triangle(bridge,vx,vy,vz,scalefac);
	bridge=set_trianglecolor(bridge,0.4,0.2,0.0,1);

	vx[0]=-4; vy[0]=4; vz[0]=0;
	vx[1]=-4; vy[1]=-4; vz[1]=0;
	vx[2]=-2; vy[2]=-4; vz[2]=0;
	bridge=add_triangle(bridge,vx,vy,vz,scalefac);
	vx[0]=-2; vy[0]=4; vz[0]=0;
	vx[1]=-4; vy[1]=4; vz[1]=0;
	vx[2]=-2; vy[2]=-4; vz[2]=0;
	bridge=add_triangle(bridge,vx,vy,vz,scalefac);
	bridge=set_trianglecolor(bridge,0.5,0.3,0.1,1);

	vx[0]=-2; vy[0]=4; vz[0]=0;
	vx[1]=-2; vy[1]=-4; vz[1]=0;
	vx[2]=-1.5; vy[2]=-4; vz[2]=-0.2;
	bridge=add_triangle(bridge,vx,vy,vz,scalefac);
	vx[0]=-1.5; vy[0]=4; vz[0]=-0.2;
	vx[1]=-2; vy[1]=4; vz[1]=0;
	vx[2]=-1.5; vy[2]=-4; vz[2]=-0.2;
	bridge=add_triangle(bridge,vx,vy,vz,scalefac);
	bridge=set_trianglecolor(bridge,0.55,0.35,0.1,1);

	// 2e
	vx[0]=-4.5+5; vy[0]=4; vz[0]=-0.2;
	vx[1]=-4.5+5; vy[1]=-4; vz[1]=-0.2;
	vx[2]=-4+5; vy[2]=-4; vz[2]=0;
	bridge=add_triangle(bridge,vx,vy,vz,scalefac);
	vx[0]=-4+5; vy[0]=4; vz[0]=0;
	vx[1]=-4.5+5; vy[1]=4; vz[1]=-0.2;
	vx[2]=-4+5; vy[2]=-4; vz[2]=0;
	bridge=add_triangle(bridge,vx,vy,vz,scalefac);
	bridge=set_trianglecolor(bridge,0.4,0.2,0.0,1);

	vx[0]=-4+5; vy[0]=4; vz[0]=0;
	vx[1]=-4+5; vy[1]=-4; vz[1]=0;
	vx[2]=-2+5; vy[2]=-4; vz[2]=0;
	bridge=add_triangle(bridge,vx,vy,vz,scalefac);
	vx[0]=-2+5; vy[0]=4; vz[0]=0;
	vx[1]=-4+5; vy[1]=4; vz[1]=0;
	vx[2]=-2+5; vy[2]=-4; vz[2]=0;
	bridge=add_triangle(bridge,vx,vy,vz,scalefac);
	bridge=set_trianglecolor(bridge,0.5,0.3,0.1,1);

	vx[0]=-2+5; vy[0]=4; vz[0]=0;
	vx[1]=-2+5; vy[1]=-4; vz[1]=0;
	vx[2]=-1.5+5; vy[2]=-4; vz[2]=-0.2;
	bridge=add_triangle(bridge,vx,vy,vz,scalefac);
	vx[0]=-1.5+5; vy[0]=4; vz[0]=-0.2;
	vx[1]=-2+5; vy[1]=4; vz[1]=0;
	vx[2]=-1.5+5; vy[2]=-4; vz[2]=-0.2;
	bridge=add_triangle(bridge,vx,vy,vz,scalefac);
	bridge=set_trianglecolor(bridge,0.55,0.35,0.1,1);

	// 3e
	vx[0]=-5; vy[0]=3.5; vz[0]=0;
	vx[1]=-5; vy[1]=3.2; vz[1]=0.2;
	vx[2]=5; vy[2]=3.2; vz[2]=0.2;
	bridge=add_triangle(bridge,vx,vy,vz,scalefac);
	vx[0]=-5; vy[0]=3.5; vz[0]=0;
	vx[1]=5; vy[1]=3.2; vz[1]=0.2;
	vx[2]=5; vy[2]=3.5; vz[2]=0;
	bridge=add_triangle(bridge,vx,vy,vz,scalefac);
	bridge=set_trianglecolor(bridge,0.5,0.3,0.1,1);

	vx[0]=-5; vy[0]=3.2; vz[0]=0.2;
	vx[1]=-5; vy[1]=2.9; vz[1]=0;
	vx[2]=5; vy[2]=2.9; vz[2]=0;
	bridge=add_triangle(bridge,vx,vy,vz,scalefac);
	vx[0]=-5; vy[0]=3.2; vz[0]=0.2;
	vx[1]=5; vy[1]=2.9; vz[1]=0;
	vx[2]=5; vy[2]=3.2; vz[2]=0.2;
	bridge=add_triangle(bridge,vx,vy,vz,scalefac);
	bridge=set_trianglecolor(bridge,0.6,0.4,0.1,1);

	// 4e
	vx[0]=-5; vy[0]=3.5-6; vz[0]=0;
	vx[1]=-5; vy[1]=3.2-6; vz[1]=0.2;
	vx[2]=5; vy[2]=3.2-6; vz[2]=0.2;
	bridge=add_triangle(bridge,vx,vy,vz,scalefac);
	vx[0]=-5; vy[0]=3.5-6; vz[0]=0;
	vx[1]=5; vy[1]=3.2-6; vz[1]=0.2;
	vx[2]=5; vy[2]=3.5-6; vz[2]=0;
	bridge=add_triangle(bridge,vx,vy,vz,scalefac);
	bridge=set_trianglecolor(bridge,0.5,0.3,0.1,1);

	vx[0]=-5; vy[0]=3.2-6; vz[0]=0.2;
	vx[1]=-5; vy[1]=2.9-6; vz[1]=0;
	vx[2]=5; vy[2]=2.9-6; vz[2]=0;
	bridge=add_triangle(bridge,vx,vy,vz,scalefac);
	vx[0]=-5; vy[0]=3.2-6; vz[0]=0.2;
	vx[1]=5; vy[1]=2.9-6; vz[1]=0;
	vx[2]=5; vy[2]=3.2-6; vz[2]=0.2;
	bridge=add_triangle(bridge,vx,vy,vz,scalefac);
	bridge=set_trianglecolor(bridge,0.6,0.4,0.1,1);

	return bridge;
}

TRIANGLEINFO *create_abombman_body(double scalefac)
{
	TRIANGLEINFO *abombman_body;
	int i, layer, step;
	double vx[3], vy[3], vz[3];
	float red,green,blue;
	double radius1[] = { 2.0, 3.0, 3.5, 3.7, 3.6, 3.0, 2.0 }; 
	double radius2[] = { 3.0, 3.5, 3.7, 3.6, 3.0, 2.0, 1.5 };
	double ypos1[] = {  1.0,  0.0, -1.5, -3.5, - 6.0, -10.0, -14.0 };
	double ypos2[] = {  0.0, -1.5, -3.5, -6.0, -10.0, -14.0, -15.0 };

	abombman_body=NULL;
	red=0;
	green=0;
	blue=0.75;
	step=5760/(8+2*model_detail);
	for (i=0; i<8+2*model_detail; i++)
	{
		vx[0]=get_x(i*step,2);
		vy[0]=get_y(i*step,2);
		vz[0]=ypos1[0];
		vx[1]=0;
		vy[1]=0;
		vz[1]=ypos1[0]+0.75;
		if (i==8+2*model_detail-1)
		{
			vx[2]=get_x(0,2);
			vy[2]=get_y(0,2);
		} else
		{
			vx[2]=get_x((i*step+step)%5760,2);
			vy[2]=get_y((i*step+step)%5760,2);
		}
		vz[2]=ypos1[0];
		abombman_body=add_triangle(abombman_body,vx,vy,vz,scalefac);
		abombman_body=set_trianglecolor(abombman_body,red,green,blue,1);
		if (blue!=0.5) blue=0.5; else blue=0.75;
	}
	for (layer=0; layer<7; layer++)
	{
		if (blue!=0.5) blue=0.5; else blue=0.75;
		for (i=0; i<8+2*model_detail; i++)
		{
			vx[0]=get_x(i*step,radius2[layer]);
			vy[0]=get_y(i*step,radius2[layer]);
			vz[0]=ypos2[layer];
			vx[1]=get_x(i*step,radius1[layer]);
			vy[1]=get_y(i*step,radius1[layer]);
			vz[1]=ypos1[layer];
			if (i==8+2*model_detail-1)
			{
				vx[2]=get_x(0,radius2[layer]);
				vy[2]=get_y(0,radius2[layer]);
			} else
			{
				vx[2]=get_x((i*step+step)%5760,radius2[layer]);
				vy[2]=get_y((i*step+step)%5760,radius2[layer]);
			}
			vz[2]=ypos2[layer];
			abombman_body=add_triangle(abombman_body,vx,vy,vz,scalefac);

			vx[0]=get_x(i*step,radius1[layer]);
			vy[0]=get_y(i*step,radius1[layer]);
			vz[0]=ypos1[layer];
			if (i==8+2*model_detail-1)
			{
				vx[1]=get_x(0,radius1[layer]);
				vy[1]=get_y(0,radius1[layer]);
				vz[1]=ypos1[layer];
				vx[2]=get_x(0,radius2[layer]);
				vy[2]=get_y(0,radius2[layer]);
				vz[2]=ypos2[layer];
			} else
			{
				vx[1]=get_x((i*step+step)%5760,radius1[layer]);
				vy[1]=get_y((i*step+step)%5760,radius1[layer]);
				vz[1]=ypos1[layer];
				vx[2]=get_x((i*step+step)%5760,radius2[layer]);
				vy[2]=get_y((i*step+step)%5760,radius2[layer]);
				vz[2]=ypos2[layer];
			}
			abombman_body=add_triangle(abombman_body,vx,vy,vz,scalefac);

			abombman_body=set_trianglecolor(abombman_body,red,green,blue,1);
			if (blue!=0.5) blue=0.5; else blue=0.75;
		}
	}
	blue=0;
	red=0.75;
	for (i=0; i<5760; i+=1920)
	{
		vx[0]=get_x(i,radius1[5]);
		vy[0]=get_y(i,radius1[5]);
		vz[0]=ypos1[5];
		vx[2]=get_x((i+5760-160)%5760,radius2[5]);
		vy[2]=get_y((i+5760-160)%5760,radius2[5]);
		vz[2]=ypos2[5];
		vx[1]=get_x(i,radius1[5]+1);
		vy[1]=get_y(i,radius1[5]+1);
		vz[1]=ypos2[6];
		abombman_body=add_triangle(abombman_body,vx,vy,vz,scalefac);
		abombman_body=set_trianglecolor(abombman_body,red,green,blue,1);

		vx[0]=get_x(i,radius1[5]);
		vy[0]=get_y(i,radius1[5]);
		vz[0]=ypos1[5];
		vx[1]=get_x((i+160)%5760,radius2[5]);
		vy[1]=get_y((i+160)%5760,radius2[5]);
		vz[1]=ypos2[5];
		vx[2]=get_x(i,radius1[5]+1);
		vy[2]=get_y(i,radius1[5]+1);
		vz[2]=ypos2[6];
		abombman_body=add_triangle(abombman_body,vx,vy,vz,scalefac);
		abombman_body=set_trianglecolor(abombman_body,red,green,blue,1);
	}

	return abombman_body;
}

TRIANGLEINFO *create_monster(double scalefac)
{
	TRIANGLEINFO *abombman_body;
	int i, layer, step;
	double vx[3], vy[3], vz[3];
	float color;
	double radius1[] = { 2.0, 3.0, 3.5, 3.7, 3.6, 4.0, 4.2 }; 
	double radius2[] = { 3.0, 3.5, 3.7, 3.6, 4.0, 4.2, 5 };
	double ypos1[] = {  1.0+14.4,  0.0+14.4, -1.5+14.4, -3.5+14.4, - 6.0+14.4, -10.0+14.4, -12.0+14.4 };
	double ypos2[] = {  0.0+14.4, -1.5+14.4, -3.5+14.4, -6.0+14.4, -10.0+14.4, -12.0+14.4, -15.0+14.4 };

	abombman_body=NULL;
	color=1;
	step=5760/(8+2*model_detail);
	for (i=0; i<8+2*model_detail; i++)
	{
		vx[0]=get_x(i*step,2);
		vy[0]=get_y(i*step,2);
		vz[0]=ypos1[0];
		vx[1]=0;
		vy[1]=0;
		vz[1]=ypos1[0]+0.75;
		if (i==8+2*model_detail-1)
		{
			vx[2]=get_x(0,2);
			vy[2]=get_y(0,2);
		} else
		{
			vx[2]=get_x((i*step+step)%5760,2);
			vy[2]=get_y((i*step+step)%5760,2);
		}
		vz[2]=ypos1[0];
		abombman_body=add_triangle(abombman_body,vx,vy,vz,scalefac);
		abombman_body=set_trianglecolor(abombman_body,color,color,color,0.5);
		if (color!=1) color=1; else color=0.95;
	}
	for (layer=0; layer<7; layer++)
	{
		if (color!=1) color=1; else color=0.95;
		for (i=0; i<8+2*model_detail; i++)
		{
			vx[0]=get_x(i*step,radius2[layer]);
			vy[0]=get_y(i*step,radius2[layer]);
			vz[0]=ypos2[layer];
			vx[1]=get_x(i*step,radius1[layer]);
			vy[1]=get_y(i*step,radius1[layer]);
			vz[1]=ypos1[layer];
			if (i==8+2*model_detail-1)
			{
				vx[2]=get_x(0,radius2[layer]);
				vy[2]=get_y(0,radius2[layer]);
			} else
			{
				vx[2]=get_x((i*step+step)%5760,radius2[layer]);
				vy[2]=get_y((i*step+step)%5760,radius2[layer]);
			}
			vz[2]=ypos2[layer];
			if (layer<6)
				abombman_body=add_triangle(abombman_body,vx,vy,vz,scalefac);

			vx[0]=get_x(i*step,radius1[layer]);
			vy[0]=get_y(i*step,radius1[layer]);
			vz[0]=ypos1[layer];
			if (i==8+2*model_detail-1)
			{
				vx[1]=get_x(0,radius1[layer]);
				vy[1]=get_y(0,radius1[layer]);
				vz[1]=ypos1[layer];
				vx[2]=get_x(0,radius2[layer]);
				vy[2]=get_y(0,radius2[layer]);
				vz[2]=ypos2[layer];
			} else
			{
				vx[1]=get_x((i*step+step)%5760,radius1[layer]);
				vy[1]=get_y((i*step+step)%5760,radius1[layer]);
				vz[1]=ypos1[layer];
				vx[2]=get_x((i*step+step)%5760,radius2[layer]);
				vy[2]=get_y((i*step+step)%5760,radius2[layer]);
				vz[2]=ypos2[layer];
			}
			abombman_body=add_triangle(abombman_body,vx,vy,vz,scalefac);

			if (layer==2 && (i==0 || i==2))
				abombman_body=set_trianglecolor(abombman_body,color,0,0,0.5);
			else
				abombman_body=set_trianglecolor(abombman_body,color,color,color,0.5);
			if (color!=1) color=1; else color=0.95;
		}
	}

	return abombman_body;
}

TRIANGLEINFO *create_floor_tile(double scalefac, double brightness, double depth)
{
	TRIANGLEINFO *floor_tile;
	double vx[3], vy[3], vz[3];

	floor_tile=NULL;	

	vx[0]=-4; vy[0]=4; vz[0]=0;
	vx[1]=-4; vy[1]=-4; vz[1]=0;
	vx[2]=4; vy[2]=4; vz[2]=0;
	floor_tile=add_triangle(floor_tile,vx,vy,vz,scalefac);
	vx[0]=4; vy[0]=4; vz[0]=0;
	vx[1]=-4; vy[1]=-4; vz[1]=0;
	vx[2]=4; vy[2]=-4; vz[2]=0;
	floor_tile=add_triangle(floor_tile,vx,vy,vz,scalefac);
	floor_tile=set_trianglecolor(floor_tile,0.2*brightness,0.6*brightness,0.2*brightness,1);

	vx[0]=-5; vy[0]=5; vz[0]=-0.5;
	vx[1]=-5; vy[1]=-5; vz[1]=-0.5;
	vx[2]=-4; vy[2]=4; vz[2]=0;
	floor_tile=add_triangle(floor_tile,vx,vy,vz,scalefac);
	vx[0]=-5; vy[0]=-5; vz[0]=-0.5;
	vx[1]=-4; vy[1]=-4; vz[1]=0;
	vx[2]=-4; vy[2]=4; vz[2]=0;
	floor_tile=add_triangle(floor_tile,vx,vy,vz,scalefac);
	floor_tile=set_trianglecolor(floor_tile,0.2*brightness,0.5*brightness,0.2*brightness,1);

	vx[0]=4; vy[0]=4; vz[0]=0;
	vx[1]=4; vy[1]=-4; vz[1]=0;
	vx[2]=5; vy[2]=5; vz[2]=-0.5;
	floor_tile=add_triangle(floor_tile,vx,vy,vz,scalefac);
	vx[0]=5; vy[0]=5; vz[0]=-0.5;
	vx[1]=4; vy[1]=-4; vz[1]=0;
	vx[2]=5; vy[2]=-5; vz[2]=-0.5;
	floor_tile=add_triangle(floor_tile,vx,vy,vz,scalefac);
	floor_tile=set_trianglecolor(floor_tile,0.2*brightness,0.65*brightness,0.2*brightness,1);

	vx[0]=-5; vy[0]=5; vz[0]=-0.5;
	vx[1]=-4; vy[1]=4; vz[1]=0;
	vx[2]=5; vy[2]=5; vz[2]=-0.5;
	floor_tile=add_triangle(floor_tile,vx,vy,vz,scalefac);
	vx[0]=5; vy[0]=5; vz[0]=-0.5;
	vx[1]=-4; vy[1]=4; vz[1]=0;
	vx[2]=4; vy[2]=4; vz[2]=0;
	floor_tile=add_triangle(floor_tile,vx,vy,vz,scalefac);
	floor_tile=set_trianglecolor(floor_tile,0.2*brightness,0.4*brightness,0.2*brightness,1);

	vx[0]=-4; vy[0]=-4; vz[0]=0;
	vx[1]=-5; vy[1]=-5; vz[1]=-0.5;
	vx[2]=4; vy[2]=-4; vz[2]=0;
	floor_tile=add_triangle(floor_tile,vx,vy,vz,scalefac);
	vx[0]=4; vy[0]=-4; vz[0]=0;
	vx[1]=-5; vy[1]=-5; vz[1]=-0.5;
	vx[2]=5; vy[2]=-5; vz[2]=-0.5;
	floor_tile=add_triangle(floor_tile,vx,vy,vz,scalefac); 
	floor_tile=set_trianglecolor(floor_tile,0.2*brightness,0.75*brightness,0.2*brightness,1);

	if (model_detail>=4 && depth<0)
	{
		vx[0]=-5; vy[0]=-5; vz[0]=-0.5;
		vx[1]=-5; vy[1]=-5; vz[1]=depth;
		vx[2]=5; vy[2]=-5; vz[2]=-0.5;
		floor_tile=add_triangle(floor_tile,vx,vy,vz,scalefac);
		vx[0]=5; vy[0]=-5; vz[0]=-0.5;
		vx[1]=-5; vy[1]=-5; vz[1]=depth;
		vx[2]=5; vy[2]=-5; vz[2]=depth;
		floor_tile=add_triangle(floor_tile,vx,vy,vz,scalefac);
		floor_tile=set_trianglecolor(floor_tile,0.5,0.5,0,1);

		vx[0]=5; vy[0]=-5; vz[0]=-0.5;
		vx[1]=5; vy[1]=-5; vz[1]=depth;
		vx[2]=5; vy[2]=5; vz[2]=depth;
		floor_tile=add_triangle(floor_tile,vx,vy,vz,scalefac);
		vx[0]=5; vy[0]=-5; vz[0]=-0.5;
		vx[1]=5; vy[1]=5; vz[1]=depth;
		vx[2]=5; vy[2]=5; vz[2]=-0.5;
		floor_tile=add_triangle(floor_tile,vx,vy,vz,scalefac);
		floor_tile=set_trianglecolor(floor_tile,0.4,0.4,0,1);

		vx[0]=-5; vy[0]=5; vz[0]=-0.5;
		vx[1]=-5; vy[1]=-5; vz[1]=depth;
		vx[2]=-5; vy[2]=-5; vz[2]=-0.5;
		floor_tile=add_triangle(floor_tile,vx,vy,vz,scalefac);
		vx[0]=-5; vy[0]=5; vz[0]=-0.5;
		vx[1]=-5; vy[1]=5; vz[1]=depth;
		vx[2]=-5; vy[2]=-5; vz[2]=depth;
		floor_tile=add_triangle(floor_tile,vx,vy,vz,scalefac);
		floor_tile=set_trianglecolor(floor_tile,0.3,0.3,0,1);

		vx[0]=-5; vy[0]=5; vz[0]=-0.5;
		vx[1]=5; vy[1]=5; vz[1]=depth;
		vx[2]=-5; vy[2]=5; vz[2]=depth;
		floor_tile=add_triangle(floor_tile,vx,vy,vz,scalefac);
		vx[0]=-5; vy[0]=5; vz[0]=-0.5;
		vx[1]=5; vy[1]=5; vz[1]=-0.5;
		vx[2]=5; vy[2]=5; vz[2]=depth;
		floor_tile=add_triangle(floor_tile,vx,vy,vz,scalefac);
		floor_tile=set_trianglecolor(floor_tile,0.2,0.2,0,1);

	}

	return floor_tile;
}

TRIANGLEINFO *set_trampoline_depth(TRIANGLEINFO *trampoline, double depth)
{
	TRIANGLEINFO *start;

	start=trampoline;
	while (start!=NULL)
	{
		start->z[1]=depth;
		start=start->nexttriangle;
	}

	return trampoline;
}

TRIANGLEINFO *create_trampoline(double scalefac, double depth, double color1[3], double color2[3])
{
	TRIANGLEINFO *trampoline;
	double vx[3], vy[3], vz[3];
	int i,step;
	double color;

	trampoline=NULL;	

	color=1;
	step=5760/(8+2*model_detail);
	for (i=0; i<8+2*model_detail; i++)
	{
		vx[0]=get_x(i*step,4);
		vy[0]=get_y(i*step,4);
		vz[0]=0;
		vx[1]=0;
		vy[1]=0;
		vz[1]=0;
		if (i==8+2*model_detail-1)
		{
			vx[2]=get_x(0,4);
			vy[2]=get_y(0,4);
		} else
		{
			vx[2]=get_x((i*step+step)%5760,4);
			vy[2]=get_y((i*step+step)%5760,4);
		}
		vz[2]=0;
		trampoline=add_triangle(trampoline,vx,vy,vz,scalefac);
		if (color==1)
			trampoline=set_trianglecolor(trampoline,color1[0],color1[1],color1[2],1);
		else
			trampoline=set_trianglecolor(trampoline,color2[0],color2[1],color2[2],1);
		color=1-color;
	}
	trampoline=set_trampoline_depth(trampoline,depth);

	return trampoline;
}

TRIANGLEINFO *create_shadow(double scalefac)
{
	TRIANGLEINFO *shadow;
	double vx[3], vy[3], vz[3];
	int i,step;

	shadow=NULL;	

	step=5760/(8+2*model_detail);
	for (i=0; i<8+2*model_detail; i++)
	{
		vx[0]=get_x(i*step,4);
		vy[0]=get_y(i*step,4);
		vz[0]=0;
		vx[1]=0;
		vy[1]=0;
		vz[1]=0;
		if (i==8+2*model_detail-1)
		{
			vx[2]=get_x(0,4);
			vy[2]=get_y(0,4);
		} else
		{
			vx[2]=get_x((i*step+step)%5760,4);
			vy[2]=get_y((i*step+step)%5760,4);
		}
		vz[2]=0;
		shadow=add_triangle(shadow,vx,vy,vz,scalefac);
	}
	shadow=set_trianglecolor(shadow,0,0,0,0.65);

	return shadow;
}

TRIANGLEINFO *create_dot(double scalefac, double color[3])
{
	TRIANGLEINFO *dot;
	double vx[3], vy[3], vz[3];
	int i,step;

	dot=NULL;	

	step=5760/12;
	for (i=0; i<12; i++)
	{
		vx[0]=get_x(i*step,2);
		vy[0]=get_y(i*step,2);
		vz[0]=0;
		vx[1]=0;
		vy[1]=0;
		vz[1]=0.5;
		if (i==11)
		{
			vx[2]=get_x(0,2);
			vy[2]=get_y(0,2);
		} else
		{
			vx[2]=get_x((i*step+step)%5760,2);
			vy[2]=get_y((i*step+step)%5760,2);
		}
		vz[2]=0;
		dot=add_triangle(dot,vx,vy,vz,scalefac);
	}
	dot=set_trianglecolor(dot,color[0],color[1],color[2],0.3);

	return dot;
}

TRIANGLEINFO *create_numbers(double scalefac, char *number)
{
	TRIANGLEINFO *dot;
	double vx[3], vy[3], vz[3];
	BITMAP *temp;
	int x,y,i,step;

	temp=create_bitmap(20,20);
	clear_to_color(temp,0);
	textout(temp,font,number,0,0,makecol(255,255,255));
	dot=NULL;	
	for (y=0; y<20; y++)
		for (x=0; x<20; x++)
			if (getpixel(temp,x,19-y)!=0)
			{
				step=5760/6;
				for (i=0; i<6; i++)
				{
					vx[0]=get_x(i*step,2)+(double)(x*110*scalefac);
					vy[0]=get_y(i*step,2)+(double)(y*110*scalefac);
					vz[0]=0;
					vx[1]=(double)(x*110*scalefac);
					vy[1]=(double)(y*110*scalefac);
					vz[1]=0.5;
					if (i==5)
					{
						vx[2]=get_x(0,2)+(double)(x*110*scalefac);
						vy[2]=get_y(0,2)+(double)(y*110*scalefac);
					} else
					{
						vx[2]=get_x((i*step+step)%5760,2)+(double)(x*110*scalefac);
						vy[2]=get_y((i*step+step)%5760,2)+(double)(y*110*scalefac);
					}
					vz[2]=0;
					dot=add_triangle(dot,vx,vy,vz,scalefac);
				}
			}
	dot=set_trianglecolor(dot,1,1,1,0.5);

	destroy_bitmap(temp);
	return dot;
}

void create_trianglelists(void)
{
	double color1[3], color2[3];
//	int prev_model_detail;
	
	abombman_body=create_abombman_body(0.25);
	floor_tile[0]=create_floor_tile(0.5,1,-60);
	floor_tile[1]=create_floor_tile(0.5,0.7,-60);
//	prev_model_detail=model_detail;
//	model_detail=0;
	floor_tile[2]=create_floor_tile(0.5,1,0);
	floor_tile[3]=create_floor_tile(0.5,0.7,0);
//	model_detail=prev_model_detail;

	floor_tile[4]=create_floor_tile(0.5,1,-4);
	floor_tile[5]=create_floor_tile(0.5,0.7,-4);

	floor_tile[6]=create_floor_tile(0.5,1,-26);
	floor_tile[7]=create_floor_tile(0.5,0.7,-26);

	color1[0]=1; color1[1]=1; color1[2]=1;
	color2[0]=0.75; color2[1]=0.75; color2[2]=0.75;
	trampoline[0]=create_trampoline(0.5,-0.5,color1,color2);

	color1[0]=1; color1[1]=1; color1[2]=0;
	color2[0]=0.75; color2[1]=0.75; color2[2]=0;
	trampoline[1]=create_trampoline(0.5,-0.5,color1,color2);

	color1[0]=1; color1[1]=0.5; color1[2]=0;
	color2[0]=0.75; color2[1]=0.25; color2[2]=0;
	trampoline[2]=create_trampoline(0.5,-0.5,color1,color2);

	color1[0]=1; color1[1]=0; color1[2]=0;
	color2[0]=0.75; color2[1]=0; color2[2]=0;
	trampoline[3]=create_trampoline(0.5,-0.5,color1,color2);

	food=create_food(0.5);

	bridge[0]=create_bridge(0.5);
	bridge[1]=create_bridge(0.5);

	shadow=create_shadow(0.25);

	ghost=create_monster(0.25);

	color1[0]=1; color1[1]=0; color1[2]=0;
	dots[0]=create_dot(0.03,color1);

	color1[0]=0; color1[1]=0; color1[2]=1;
	dots[1]=create_dot(0.03,color1);

	color1[0]=1; color1[1]=1; color1[2]=0;
	dots[2]=create_dot(0.03,color1);

	numbers[0]=create_numbers(0.035,"0");
	numbers[1]=create_numbers(0.035,"1");
	numbers[2]=create_numbers(0.035,"2");
	numbers[3]=create_numbers(0.035,"3");
	numbers[4]=create_numbers(0.035,"4");
	numbers[5]=create_numbers(0.035,"5");
	numbers[6]=create_numbers(0.035,"6");
	numbers[7]=create_numbers(0.035,"7");
	numbers[8]=create_numbers(0.035,"8");
	numbers[9]=create_numbers(0.035,"9");
}

void destroy_trianglelists(void)
{
	free_trianglelist(abombman_body);
	free_trianglelist(floor_tile[0]);
	free_trianglelist(floor_tile[1]);
	free_trianglelist(floor_tile[2]);
	free_trianglelist(floor_tile[3]);
	free_trianglelist(floor_tile[4]);
	free_trianglelist(floor_tile[5]);
	free_trianglelist(floor_tile[6]);
	free_trianglelist(floor_tile[7]);
	free_trianglelist(trampoline[0]);
	free_trianglelist(trampoline[1]);
	free_trianglelist(trampoline[2]);
	free_trianglelist(trampoline[3]);
	free_trianglelist(food);
	free_trianglelist(bridge[0]);
	free_trianglelist(bridge[1]);
	free_trianglelist(shadow);
	free_trianglelist(ghost);
	free_trianglelist(dots[0]);
	free_trianglelist(dots[1]);
	free_trianglelist(dots[2]);
	free_trianglelist(numbers[0]);
	free_trianglelist(numbers[1]);
	free_trianglelist(numbers[2]);
	free_trianglelist(numbers[3]);
	free_trianglelist(numbers[4]);
	free_trianglelist(numbers[5]);
	free_trianglelist(numbers[6]);
	free_trianglelist(numbers[7]);
	free_trianglelist(numbers[8]);
	free_trianglelist(numbers[9]);
}

